/* parser.c -- Function[s] to parse /etc/mprl.conf
   Copyright (C) 2003, 2004
   Arturo "Buanzo" Busleiman <buanzo@buanzo.com.ar> <buanzo@linux.org.ar>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "parser.h"
#include "buennoc.h"

struct proto_struct *
parse_mprl_conf (char *which, int *error_code)
{
  struct proto_struct *ldef;
  char *mcf_line;
  char *ttok;
  int i = 0;
  char *rest;
  FILE *mcf;
  if ((mcf = fopen ("/etc/mprl.conf", "r")) == NULL)
    {
      *error_code = ERR_PARSER_OPEN_CONFIG;
      return (NULL);
    }
ldef=buennoc(1024,0);
  if (which == NULL)
    {
      *error_code = ERR_PARSER_CANNOT_ALLOC;
      return (ldef);
    }

  mcf_line = buennoc (255,0);
  while (!feof (mcf))
    {
      fgets (mcf_line, 254, mcf);
      if (strncmp (which, mcf_line, strlen (which)) == 0)
	{
	  mcf_line[strlen (mcf_line) - 1] = 0;
/* protocol is defined. Get it's default port and args. Fill struct. */
	  ldef->name = buennoc (strlen (which) - 1,0);
memset(ldef->name,0,sizeof(ldef->name));

	  strncpy (ldef->name, which, strlen (which) - 1);
	  rest = mcf_line + strlen (which);
	  if (rest[0] == ':')
	    {
	      rest += 1;
	      ldef->default_port = NULL;
	    }
	  else
	    {
	      for (i = 1; i <= 5; i++) 
		if (rest[i] == ':')
		  break;
if (i==6) {
printf("\033[1;32mError!\033[0;37m Check default_port value for protocol '%s'.\n",ldef->name);
sleep(3);
exit(240);
}

	      ldef->default_port = buennoc (i,0);
	      memset(ldef->default_port,0,sizeof(ldef->default_port));
	      strncpy (ldef->default_port, rest, i);
	      rest += i + 1;
	    }

/* at this point 'rest' has the command line, let's parse it token by
token, using ' ' as separator. sanitize a bit.*/

/* remove leading spaces */
	  while (1)
	    if (rest[0] == ' ')
	      rest += 1;
	    else
	      break;

/* remove trailing spaces */

	  while (1)
	    if (rest[strlen (rest) - 1] == ' ')
	      rest[strlen (rest) - 1] = 0;
	    else
	      break;

	  if (strlen (rest) == 0)
	    return (NULL);

	  ttok = strtok (rest, " ");
	  if (ttok == NULL)
	    ldef->n_args = 0;
	  else
	    {
	      ldef->args = buennoc (1024,0);
	      ldef->args[0] = buennoc (strlen (ttok),0);
	      strncpy (ldef->args[0], ttok, strlen (ttok));
	      for (i = 1;; i++)
		{
		  ttok = strtok (NULL, " ");
		  if (ttok == NULL)
		    {
		      ldef->n_args = i;
		      ldef->args[i] = buennoc (2,0);
		      ldef->args[i] = NULL;
		      break;
		    }
		  else
		    {
		      ldef->args[i] = buennoc (strlen (ttok),0);
memset(ldef->args[i],0,sizeof(ldef->args[i]));
		      strncpy (ldef->args[i], ttok, strlen (ttok)+1);
		    }
		}
	    }
	  free (mcf_line);
	  return (ldef);
	}

    }

  free (mcf_line);
  *error_code = ERR_PARSER_NOT_PROTOCOL;
  return (NULL);
}
